/************************************************************************
 *	camt3a.c       	  4-DEC1991		Y.Takeuchi (TIT)	*
 *									*
 *	Use plock()							*
 *	CAMAC LAM Interrupt speed test					*
 *	CAMAC Scaler ch1: input 1MHz pulse				*
 *	CAMAC ADC							*
 *	camt3 [lam_loop_count [list_file_name]]				*
 ************************************************************************/
#include <stdio.h>
#include <sys/lock.h>
#include "camtest.h"
#include "camlib.h"

#define DEFLOP    10
#define MAXLOP    100
#define DEFNAME   "camt3.obj"
#define MAXLIST   2000
#define MAXDATA   100000
#define LAMTOUT   2             /* 2 sec */

main(argc, argv)
int argc;
char *argv[];
{
    int loop, i, mask, nafclrlam, nafct, nafrt, q, x, dat0, dat1, dumy;
    int lenlis, data[MAXDATA], nafr, nafc;
    struct cc_iosb iosb;
    char filename[33];
    unsigned short list[MAXLIST];

/***** Lock process in memory *****/
    if (plock(PROCLOCK)) {
	perror("plock");
	exit(1);
    }
/**********************************/

/* Read options */
    loop = (argc > 1) ? atoi(argv[1]) : DEFLOP;
    strcpy(filename, (argc > 2) ? argv[2] : DEFNAME);

/* Initialize valiable */
    mask = 1 << (CAMAC_ADC - 1);
    nafct = NAF(CAMAC_SCALER, 1, 9);
    nafrt = NAF(CAMAC_SCALER, 1, 0);
    nafc = NAF(CAMAC_SCALER, 2, 9);
    nafr = NAF(CAMAC_SCALER, 2, 0);
    nafclrlam = NAF(CAMAC_ADC, 0, 10);

/* Open and Initialize CAMAC */
    if (CAMOPN()) {
      perror("CAMOPN");
      exit(1);
  }
    CSETCR(0);
    CGENC();
    CGENZ();
    CREMI();

/* Read and Install CAMAC list */
    if (CRLIST(filename, list, MAXLIST, &lenlis, &i)) {
	perror("CRLIST");
        exit(2);
    }
    if (CLLOAD(list, lenlis, &i)) {
	perror("CLLOAD");
        exit(3);
    }

/**** Measurement ****/
/* time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++) {
	CAMAC(nafct, &dumy, &q, &x);
	CAMAC(nafrt, &dat0, &q, &x);
	CAMAC(nafrt, &dat1, &q, &x);
	printf("%d\n", dat1 - dat0);
    }

/* Call cc-driver */
    CELAM(mask);

    CWLAM(LAMTOUT);
    CAMAC(nafr, &data[i], &q, &x);
    CAMAC(nafclrlam, &dumy, &q, &x);
    dumy = 1;
    while (dumy != 0) {
	CAMAC(nafc, &dumy, &q, &x);
	CAMAC(nafr, &dumy, &q, &x);
    }

    for (i = 0; i < loop; i++) {
	CWLAM(LAMTOUT);
	CAMAC(nafr, &data[i], &q, &x);
	CAMAC(nafclrlam, &dumy, &q, &x);
	dumy = 1;
	while (dumy != 0) {
	    CAMAC(nafc, &dumy, &q, &x);
	    CAMAC(nafr, &dumy, &q, &x);
	}
    }
    CDLAM();
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
      printf("%d\n", data[i]);

/* Execute list */
    iosb.s_reg = loop;
    if (CEWAI(data, MAXDATA, &iosb)) {
	perror("CEWAI");
	exit(10);
    }

/* 24bit time=0 */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", data[(i*2)+1] - data[i*2]);

/* Interrupt speed */
    printf("%d\n", loop);
    for (i = 0; i < loop; i++)
	printf("%d\n", data[loop*2+i+1]);

/* End */
    CAM_Close();
}
